// Craig S. Kaplan
//
// HYPERSPACE, BABY!
// 
// Draw a star field that's slowly moving towards the camera.
// When a star passes out of view behind the camera, "recycle"
// it by turning it into a new star off in the distance.  That
// way there's an endless field of stars flying at you.  Use
// the up and down arrow keys to control the speed.  Also,
// stretch the stars in the direction that they're moving, so that
// as the speed increases you get the classic Star Wars Hyperspace
// effect.  I'm impressed by how much bang we can get here from a 
// relatively short program.

PVector[] stars;

// The velocity, used to control how far each star moves
// after every frame.
float v = 20.0;

PVector makeStar()
{
  return new PVector( 
    random(-2*width, 2*width), 
    random(-2*height, 2*height), 
    random(-1000, 200) );
}

void setup()
{
  size( 500, 500, P3D );
  sphereDetail( 15 );
  stars = new PVector[ 2000 ];
  for ( int idx = 0; idx < stars.length; ++idx ) {
    stars[idx] = makeStar();
  }
}

void draw()
{
  background( 0 );
  fill( 255 );
  noStroke();

  for ( int idx = 0; idx < stars.length; ++idx ) {
    PVector P = stars[idx];
    pushMatrix();
    translate( P.x, P.y, P.z );
    scale( 1, 1, v/2 );
    sphere( 1 );
    popMatrix();

    P.z += v;
    if ( P.z > 500 ) {
      // If this star is no longer visible, reset it
      // to be a new star in the distance.
      stars[idx] = makeStar();
    }
  }
}

void keyPressed()
{
  if ( keyCode == UP ) {
    v *= 1.25;
  } else if ( keyCode == DOWN ) {
    v /= 1.25;
  }
}