// Craig S. Kaplan
//
// Kind of the opposite of the GridOfSpheres sketch -- draw
// an approximation of a sphere made out of a bunch of 
// little cubes.

void setup()
{
  size( 500, 500, P3D );
}

void draw()
{
  background( 50 );
  lights();
  fill( 255, 0, 0 );
  noStroke();

  randomSeed( 1 );
  
  pushMatrix();
  translate( width/2, height/2, 0 );
  rotateX( frameCount / 77.0 );
  rotateY( frameCount / 100.0 );
  scale( 30 );

  for ( int x = 0; x < 10; ++x ) {
    for ( int y = 0; y < 10; ++y ) {
      for ( int z = 0; z < 10; ++z ) {
        // Examine every cube in a 10x10x10 3D grid, but 
        // check how far that cube is from the origin and
        // only draw the ones that are within distance 5.
        // That'll work out to draw something roughly ball
        // shaped.
        float d = dist( -4.5+x, -4.5+y, -4.5+z, 0, 0, 0 );
        if ( d < 5 ) {
          pushMatrix();
          translate( -4.5 + x, -4.5 + y, -4.5 + z );
          // Setting a random seed above allows us to give
          // every cube a random colour here, and the colours
          // won't go crazy after every frame.
          fill( random(255), random(255), random(255) );
          box( 0.7, 0.7, 0.7 );
          popMatrix();
        }
      }
    }
  }

  popMatrix();
}